﻿
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Globalization;
using System.Linq;
using System.Reflection;
using System.Resources;
using System.Text;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ICP.Reporting_Outbound_MetDaan.ReportingClassLibrary;
using Infragistics.Controls.Reports;

namespace ICP.Reporting_Outbound_MetDaan.WPFReportViewer.Windows
{
    /// <summary>
    /// Interaction logic for Report_MetDaan_OPR.xaml
    /// </summary>
    public partial class Report_MetDaan_OPR : Window
    {
        public Report_MetDaan_OPR()
        {
            InitializeComponent();

            // Bij opstarten van het Window worden de korte datumnotatie en de
            // lange tijdnotatie van Windows correct gezet zodat universeel met datums
            // en tijden gewerkt wordt.
            CultureInfo ci = CultureInfo.CreateSpecificCulture(CultureInfo.CurrentCulture.Name);
            ci.DateTimeFormat.ShortDatePattern = "dd/MM/yyyy";
            ci.DateTimeFormat.DateSeparator = "-";
            ci.DateTimeFormat.LongTimePattern = "HH:mm:ss.fff";
            Thread.CurrentThread.CurrentCulture = ci;

            // De datum in DatePicker Begindatum wordt gezet op een week terug.
            datePickerBeginDatum.DisplayDate = DateTime.Today.AddDays(-7);
            datePickerBeginDatum.SelectedDate = DateTime.Today.AddDays(-7);
            // De datum in DatePicker Einddatum wordt gezet op een dag terug.
            datePickerEindDatum.DisplayDate = DateTime.Today.AddDays(-1);
            datePickerEindDatum.SelectedDate = DateTime.Today.AddDays(-1);

            // Aanroep van de nodige Methods voor het vullen van de desbetreffende ComboBoxen.
            BindComboBoxAuthoriteit(comboBoxAuthoriteit);
            BindComboBoxJaar(comboBoxJaar);
        }

        /// <summary>
        /// Methode voor het openen van een connectie naar de SQL Database "ICP" en het ophalen van de beschikbare Authorities
        /// in de SQL Tabel "Authorities".
        /// </summary>
        /// <returns>Retourneert de beschikbare Authorities in de SQL Tabel "Authorities".</returns>
        private static DataSet OpenConnectionAuthoriteit()
        {
            // Open de connection naar de SQL Database "ICP".
            string strConnectionAuthoriteit = ConfigurationManager.ConnectionStrings["ICP.Reporting_Outbound_MetDaan.WPFReportViewer.Properties.Settings.sqlDS_Authoriteit"].ConnectionString;
            SqlConnection connectionAuthoriteit = new SqlConnection(strConnectionAuthoriteit);
            // Haal de beschikbare Authorities op uit de SQL Tabel "Authorities".
            SqlDataAdapter daAuthoriteit = new SqlDataAdapter("SELECT Authority_ID, Description FROM dbo.MuChan_Labels WHERE (Description LIKE '%Outbound%') ORDER BY Description;", connectionAuthoriteit);
            DataSet dsAuthoriteit = new DataSet();
            // Vul de DataSet "dsAuthoriteit" met het resultaat uit de SELECT Statement.
            daAuthoriteit.Fill(dsAuthoriteit, "Authoriteit");
            // Geef de DataSet terug.
            return dsAuthoriteit;
        }

        /// <summary>
        /// Methode om een ComboBox te vullen met het geretourneerde resultaat uit de Methode "OpenConnectionAuthoriteit()".
        /// </summary>
        /// <param name="comboBoxName">Naam van de ComboBox die gevuld dient te worden met het geretourneerde resultaat.</param>
        public void BindComboBoxAuthoriteit(ComboBox comboBoxName)
        {
            // Gebruik de DataSet "dsAuthoriteit" uit de Methode "OpenConnectionAuthoriteit()".
            DataSet dsAuthoriteit = OpenConnectionAuthoriteit();
            // Vul de opgegeven ComboBox met de data uit de DataSet.
            comboBoxName.ItemsSource = dsAuthoriteit.Tables[0].DefaultView;
            comboBoxName.DisplayMemberPath = dsAuthoriteit.Tables[0].Columns["Description"].ToString();
            comboBoxName.SelectedValuePath = dsAuthoriteit.Tables[0].Columns["Authority_ID"].ToString();
        }

        /// <summary>
        /// Methode voor het openen van een connectie naar de SQL Database "ICP" en het ophalen van de beschikbare jaartallen
        /// in de SQL Tabel "Questionnaires_Answers".
        /// </summary>
        /// <returns>Retourneert de beschikbare jaartallen in de SQL Tabel "Questionnaires_Answers".</returns>
        private static DataSet OpenConnectionJaar()
        {
            // Open de connection naar de SQL Database "ICP".
            string strConnectionJaar = ConfigurationManager.ConnectionStrings["ICP.Reporting_Outbound_MetDaan.WPFReportViewer.Properties.Settings.sqlDS_Jaar"].ConnectionString;
            SqlConnection connectionJaar = new SqlConnection(strConnectionJaar);
            // Haal de beschikbare jaartallen op uit de SQL Tabel "Questionnaires_Answers".
            SqlDataAdapter daJaar = new SqlDataAdapter("SELECT YEAR(Record_Created_DateTime) AS Year FROM ICP.dbo.Questionnaires_Answers AS QA GROUP BY YEAR(Record_Created_DateTime) ORDER BY Year;", connectionJaar);
            DataSet dsJaar = new DataSet();
            // Vul de DataSet "dsJaar" met het resultaat uit de SELECT Statement.
            daJaar.Fill(dsJaar, "Questionnaires_Answers");
            // Geef de DataSet terug.
            return dsJaar;
        }

        /// <summary>
        /// Methode om een ComboBox te vullen met het geretourneerde resultaat uit de Methode "OpenConnectionJaar()".
        /// </summary>
        /// <param name="comboBoxName">Naam van de ComboBox die gevuld dient te worden met het geretourneerde resultaat.</param>
        public void BindComboBoxJaar(ComboBox comboBoxName)
        {
            // Gebruik de DataSet "dsJaar" uit de Methode "OpenConnectionJaar()".
            DataSet dsJaar = OpenConnectionJaar();
            // Vul de opgegeven ComboBox met de data uit de DataSet.
            comboBoxName.ItemsSource = dsJaar.Tables[0].DefaultView;
            comboBoxName.DisplayMemberPath = dsJaar.Tables[0].Columns["Year"].ToString();
            comboBoxName.SelectedValuePath = dsJaar.Tables[0].Columns["Year"].ToString();
        }

        /// <summary>
        /// Methode voor het berekenen van de eerste dag van de geselecteerde maand.
        /// </summary>
        /// <param name="iMaand">De geselecteerde maand.</param>
        /// <param name="iJaar">Het geselecteerde jaar.</param>
        /// <returns>Retourneert de eerste dag van de geselecteerde maand.</returns>
        private DateTime EersteDagMaand(int iMaand, int iJaar)
        {
            // Selecteer de eerste dag van de maand door gebruik van de DateTime class.
            DateTime dtVan = new DateTime(iJaar, iMaand, 1);

            // Verwijder alle dagen van de maand behalve de eerste.
            dtVan = dtVan.AddDays(-(dtVan.Day - 1));

            // Geef de DateTime terug.
            return dtVan;
        }

        /// <summary>
        /// Methode voor het berekenen van de laatste dag van de geselecteerde maand.
        /// </summary>
        /// <param name="iMaand">De geselecteerde maand.</param>
        /// <param name="iJaar">Het geselecteerde jaar.</param>
        /// <returns>Retourneert de laatste dag van de geselecteerde maand.</returns>
        private DateTime LaatsteDagMaand(int iMaand, int iJaar)
        {
            // Selecteer de eerste dag van de maand door gebruik van de DateTime class.
            DateTime dtTot = new DateTime(iJaar, iMaand, 1);

            // Voeg een maand toe aan de aangepaste DateTime.
            dtTot = dtTot.AddMonths(1);

            // Trek een dag af van de aangepaste DateTime.
            dtTot = dtTot.AddDays(-(dtTot.Day));

            // Geef de DateTime terug, nu tot de laatste dag van de maand gezet.
            return dtTot;
        }

        // Bij het klikken op de knop "Button_Data" worden de eerste en laatste dag van de geselecteerde maand
        // van het geselecteerde jaar berekend.
        private void Button_Data_Click(object sender, RoutedEventArgs e)
        {
            // Als de maand en/of het jaar niet geselecteerd zijn wordt een melding in een messagebox getoond.
            if (string.IsNullOrEmpty(comboBoxMaand.Text) || string.IsNullOrEmpty(comboBoxJaar.Text))
            {
                MessageBox.Show(ICP.Reporting_Outbound_MetDaan.WPFReportViewer.Resources.Pages.ParametersStrings.Binding_MessageBox_MaandJaar_Text, ICP.Reporting_Outbound_MetDaan.WPFReportViewer.Resources.Pages.ParametersStrings.Binding_MessageBox_Warning_Title, MessageBoxButton.OK, MessageBoxImage.Warning);
            }
            // Als de maand en het jaar geselecteerd zijn, vul de TextBoxen "textBoxBeginDatum" en "textBoxEindDatum"
            // en de DatePickers "datePickerBeginDatum" en "datePickerEindDatum" met de berekende begin- en einddatum.
            else
            {
                // Selecteer de maand en het jaar, klik op de "Bereken data" knop om de
                // eerste en laatste dag van de maand te berekenen van enig welke
                // maand en jaar geselecteerd in ComboBox controls.
                Int32 iJaar = 0;
                // Als het jaar in de ComboBox "comboBoxJaar" goed geselecteerd is, converteer de geselecteerde waarde naar een string.
                Boolean bSuccess = Int32.TryParse(comboBoxJaar.SelectedValue.ToString(), out iJaar);
                // Als de het jaar niet goed geselecteerd is, verlaat deze Methode.
                if (bSuccess != true) return;
                // Vul de TextBox "textBoxBeginDatum" en DatePicker "datePickerBeginDatum" met de berekende eerste dag van de maand.
                textBoxBeginDatum.Text = EersteDagMaand(comboBoxMaand.SelectedIndex + 1, iJaar).ToShortDateString();
                datePickerBeginDatum.Text = textBoxBeginDatum.Text;

                // Vul de TextBox "textBoxEindDatum" en DatePicker "datePickerEindDatum" met de berekende laatste dag van de maand.
                textBoxEindDatum.Text = LaatsteDagMaand(comboBoxMaand.SelectedIndex + 1, iJaar).ToShortDateString();
                datePickerEindDatum.Text = textBoxEindDatum.Text;
            }
        }

        // Bij het klikken op de knop "Button_VerversRapport" wordt het rapport gegenereerd en in de
        // ReportViewer weergegeven.
        private void Button_VerversRapport_Click(object sender, RoutedEventArgs e)
        {
            // Als het label en/of het script niet geselecteerd zijn wordt een melding in een messagebox getoond.
            if (string.IsNullOrEmpty(comboBoxAuthoriteit.Text))
            {
                MessageBox.Show(ICP.Reporting_Outbound_MetDaan.WPFReportViewer.Resources.Pages.ParametersStrings.Binding_MessageBox_Authoriteit_Text, ICP.Reporting_Outbound_MetDaan.WPFReportViewer.Resources.Pages.ParametersStrings.Binding_MessageBox_Warning_Title, MessageBoxButton.OK, MessageBoxImage.Warning);
            }
            // Als het label en het script geselecteerd zijn, ververs dan de ReportViewer en geef het rapport weer.
            else
            {
                this.xamReportViewer1.Refresh();
            }
        }
    }
}
